

//=============================================================================
// meshSurface.cpp 
//
//Author: Doron Nussbaum (C) 2011 All Rights Reserved.
//-----------------------------------------------------
//
// Purpose: 
//--------------
// a. demonstrates the creation of a surface and of vertex and index buffers
// b. demonstartes how to rpesent a surface 
//
//
// Description:
//--------------
//
//
//
// License
//--------------
//
// Code can be used for instructional and educational purposes and for assignments in the gaming courses at 
// the School of Compuer Science at Carleton University.
// Usage of code for other purposes is not allowed without a given permission by the author.
//
//
// Disclaimer
//--------------
//
// The code is provided as is without any warranty

//=============================================================================



#include "StdAfx.h"
#include "meshSurface.h"


// macro for release the com objects (move to a utility file)
#define COM_RELEASE(X) {if ((X) != NULL) {(X)->Release(); (X) = NULL;}}

// macro for release allocated memory
#define FREE_MEMORY(X) {if ((X) != NULL) {free(X); (X) = NULL;}}


// static class variable 

IDirect3DVertexDeclaration9* meshSurface::meshDecl = NULL;
//ID3DXEffect *meshSurface::pMeshEffect = NULL;
//D3DXHANDLE meshSurface::hWorldViewProjMat = NULL;
//D3DXHANDLE meshSurface::hTechnique = NULL;
//D3DXHANDLE meshSurface::hTime = NULL;
//D3DXHANDLE meshSurface::hHighWaves = NULL;

/******************************************************************/
/*
Purpose: meshSurface constructor


Descripton: 

Return:

*/

meshSurface::meshSurface(void)
: ind(0), 
  numVtx(0), 
  numQuads(0), 
  numTriangles(0), 
  vtx(NULL)
  , numRows(0)
  , numCols(0)
  , highWaves(0)
  , crater(0.0,0.0,0.0)
{
	createVtxDescription();
}

/******************************************************************/
/*
Purpose: meshSurface destructor


Descripton: releases all allocated memory and com objects

Return:

*/

meshSurface::~meshSurface(void)
{
	// releae the com objects
	COM_RELEASE(this->meshDecl);
	COM_RELEASE(this->mVtxBuf);
	COM_RELEASE(this->mIndBuf);
	COM_RELEASE(pMeshEffect)

	// free the memory
	FREE_MEMORY(vtx);
	FREE_MEMORY(ind);
	
}

/******************************************************************/
/*
Purpose: meshSurface constructor


Descripton: creates a surface with numRows x NumCols vertices and (numRows-1) x (NumCols-1)
quads.  Each quad size is dx x dz.  

Note that the y axis represents the height and the mesh spans the x-z axis.

Return:

*/
meshSurface::meshSurface(int numRows, int numCols, float dx, float dz) 
: ind(0), 
  numVtx(0), 
  numQuads(0), 
  numTriangles(0), 
  vtx(NULL)
  , numRows(0)
  , numCols(0)
  , highWaves(0)
  , pMeshEffect(NULL)
  , crater(0.0,0.0,0.0)

{

	setScale(1.0, 1.0, 1.0);
	createVtxDescription();
	createSurface(numRows, numCols, dx, dz);
	initEffect();
	



}

/******************************************************************/
/*
Purpose: generates the surface - the vertices and the quads


Descripton: creates a surface with numRows x NumCols vertices and (numRows-1) x (NumCols-1)
quads.  Each quad size is dx x dz.  

It allocates the required memory and store the vertices and quads with the object.  It also 
creats the required buffer on the graphics card.

The generated mesh is index based.

Note that the y axis represents the height and the mesh spans the x-z axis.

Return:
0 if success
*/


int meshSurface::createSurface(int numRows, int numCols, float dx, float dz)
{	
	int rc = 0;
	int i,j,k;
	D3DXVECTOR3 rowPos(0.0,0.0,0.0);
	D3DXVECTOR3 colPos(0.0,0.0,0.0);
	float rangeMax = 10;
	float rangeMin = 0;
	float texU, texV;	// the texture coordinates
	float texDeltaU;
	float texDeltaV;



	numVtx = numRows*numCols;
	this->numRows = numRows;
	this->numCols = numCols;
	this->dx = dx;
	this->dz = dz;

	numQuads = (numRows-1) * (numCols-1);
	numTriangles = numQuads * 2;

	// allocate memory
	vtx = (struct meshVertex *) malloc(sizeof(struct meshVertex) * numVtx);
	if (vtx == NULL) {
		// error
		rc = 1;
		goto err;
	}

	ind = (long *) malloc(sizeof(long) * numTriangles * 3);
	if (ind == NULL) {
		// error
		rc = 1;
		goto err;
	}

	// Fill the vertex buffer with positions

	// update the texture delta x
	texDeltaU = 1.0 / (this->numCols);

	// updadate the texture delata y
	texDeltaV = 1.0 / (this->numRows);

	k = 0;
	for(i = 0, texV = 1.0; i < numRows; i++, texV -= texDeltaV) {
		// assign the pos vector for all comumns in a row
		colPos = rowPos;
		for(j = 0, texU = 0.0; j < numCols; j++, texU += texDeltaU) {
			// Negate the depth coordinate to put in quadrant four.  
			// Then offset to center about coordinate system.
			vtx[k].pos =  colPos;
//			vtx[k].pos.y = (long)((double)rand()/(RAND_MAX+1) * (rangeMax - rangeMin)
//				+ rangeMin);
			vtx[k].color = D3DCOLOR_XRGB(255, 255, 0);
			colPos.x += dx;
			k++;
		}
		rowPos.z += dz;
	}


	// fill the index buffer

	k = 0;
	for(i = 0; i < numRows-1; i++) {
		for(j = 0; j < numCols-1; j++)
		{
			// fill indices for the quad
			ind[k++] = i * numCols + j;
			ind[k++] = (i+1) * numCols + j + 1;
			ind[k++] = i * numCols + j + 1;
			
			ind[k++] = i * numCols + j;
			ind[k++] = (i+1) * numCols + j;
			ind[k++] = (i+1) * numCols + j + 1;

		}
	}

	// allocate memory in the graphics card memory for the vertices and the index buffer
	createGraphicsBuffers();

	return(rc);
err:
	// clean up
	FREE_MEMORY(vtx);
	FREE_MEMORY(ind);

	return(rc);
}


/******************************************************************/
/*
Purpose: creates the required index and vertex buffers on the graphics card


Descripton: 

Return:
0 if success

*/


int meshSurface::createGraphicsBuffers(void)
{
	int rc = 0;
	struct meshVertex *v = NULL;
	long *ind = NULL;


#ifdef NUSS_SHADERS
	rc =  md3dDev->CreateVertexBuffer(this->numVtx*sizeof(struct meshVertex),D3DUSAGE_WRITEONLY,
										NULL,D3DPOOL_MANAGED,&this->mVtxBuf, NULL);
#else 
	rc =  md3dDev->CreateVertexBuffer(this->numVtx*sizeof(struct meshVertex),D3DUSAGE_WRITEONLY,
										MESH_VERTEX_FVF,D3DPOOL_MANAGED,&this->mVtxBuf, NULL);
#endif


	// lock the buffer and copy the vertices (locking the entire array
	rc = mVtxBuf->Lock(0, 0, (void **) &v, 0);
	if (rc != D3D_OK) {
		goto err;
	}

	memcpy(v, this->vtx,numVtx*sizeof(struct meshVertex));

	rc = mVtxBuf->Unlock();


	rc =  md3dDev->CreateIndexBuffer(this->numTriangles*3*sizeof(long),D3DUSAGE_WRITEONLY,
										D3DFMT_INDEX32,D3DPOOL_MANAGED,&this->mIndBuf, NULL);


	// lock the buffer and copy the vertices (locking the entire array
	rc = mIndBuf->Lock(0, 0, (void **) &ind, 0);
	if (rc != D3D_OK) {
		goto err;
	}

	memcpy(ind, this->ind,numTriangles*3*sizeof(long));

	rc = mIndBuf->Unlock();

	if (rc != D3D_OK) {
		goto err;
	}



	return 0;

err:
	return(rc);
}

/******************************************************************/
/*
Purpose: creates the vertex description interface


Descripton: 

Return:
0 if success
*/



int meshSurface::createVtxDescription(void)
{
	int rc = 0;
	// create the vertex decalation

	if (this->meshDecl == NULL) {
		struct meshVertex v;
		D3DVERTEXELEMENT9 decl[] = 
		{{0,0,D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		 {0,0,D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR, 0},
		D3DDECL_END()};
		decl[0].Offset = (char *) &v.pos - (char *) &v;
		decl[1].Offset = (char *) &v.color - (char *) &v;
		rc = md3dDev->CreateVertexDeclaration(decl, &meshDecl);
	}



	return 0;
}

/******************************************************************/
/*
Purpose: change the position of the sufce.


Descripton: set the position of hte surfce.  The srufce should be 
translated so the the vertex at (0,?,0) is moved to 
Return:

*/


int meshSurface::setCentrePosition(D3DXVECTOR3 centrePos)
{
	// to be udpated 
	this->mPosition = centrePos;
	return 0;
}

/******************************************************************/
/*
Purpose: renders the surface


Descripton: 


Return:

*/


int meshSurface::render(int time)
{

#ifdef NUSS_SHADERS
	ShaderRender(time);
#else
	TradtionalPipeRender(time);
#endif


	return 0;
}

/******************************************************************/
/*
Purpose: renders the surface


Descripton: 


Return:

*/


int meshSurface::ShaderRender(int time)
{

	int rc;
    static int angle = 0;

	static float step = 1;
	float rad = 0;
	float fTime;
	int counter = 0;

	int i=0, j=0, k=0;
	unsigned int numPasses = 0;		// containes the number of passes of the technique

	D3DXMATRIX worldMat, matTransform, matProjection, matScale, matTranslate,  matRotation;


	D3DXMatrixScaling(&matScale,mScaleX, mScaleY, mScaleZ);
	worldMat = matScale;

	D3DXMatrixTranslation(&matTranslate, mPosition.x, mPosition.y, mPosition.z);
	worldMat *= matTranslate;


	// set the source
	rc = md3dDev->SetStreamSource( 0, mVtxBuf, 0, sizeof(meshVertex) );
	rc = md3dDev->SetVertexDeclaration(meshDecl);
	// set the index buffer
	md3dDev->SetIndices(mIndBuf);


	// set the technique
	rc = pMeshEffect->SetTechnique(hFlatTechnique);
	

	// set the global matrix
	pMeshEffect->SetMatrix(hWorldMat, &worldMat);
	pMeshEffect->SetMatrix(hViewMat, &viewMat);
	pMeshEffect->SetMatrix(hProjMat, &projMat);

	// set the time
	fTime = time;
	pMeshEffect->SetFloat(hTime, fTime);
	pMeshEffect->SetFloatArray(hCrater, (float *) &crater, 3);




	// start the effect
	pMeshEffect->Begin(&numPasses, 0);		// 0 means save and restore the device states

	// start the technique passess
	for (i = 0; i < numPasses; i++) {
		pMeshEffect->BeginPass(i);

		// draw the mesh
		md3dDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, this->numVtx, 0, this->numTriangles);

		pMeshEffect->EndPass();
	}

	pMeshEffect->End();

   
	return 0;
}

/******************************************************************/
/*
Purpose: renders the surface


Descripton: 


Return:

*/


int meshSurface::TradtionalPipeRender(int time)
{

	int rc;
    static int angle = 0;

	static float step = 1;
	float rad = 0;

	static int i=0, j=0, k=0;

	D3DXMATRIX worldMat, viewMat, matTransform, matProjection, matScale, matTranslate,  matRotation;


	D3DXMatrixScaling(&matScale,mScaleX, mScaleY, mScaleZ);
	worldMat = matScale;

	D3DXMatrixTranslation(&matTranslate, mPosition.x, mPosition.y, mPosition.z);
	worldMat *= matTranslate;


	//D3DXMatrixIdentity(&worldMat);
	md3dDev->SetTransform(D3DTS_WORLD, &worldMat);

	// set the source
	rc = md3dDev->SetStreamSource( 0, mVtxBuf, 0, sizeof(meshVertex) );
#ifdef NUSS_SHADERS
	rc = md3dDev->SetVertexDeclaration(meshDecl);
#else
	rc = md3dDev->SetFVF(MESH_VERTEX_FVF);
#endif


	// set the index buffer
	md3dDev->SetIndices(mIndBuf);


	md3dDev->SetRenderState(D3DRS_COLORVERTEX, FALSE);
	md3dDev->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
	//md3dDev->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
	md3dDev->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, this->numVtx, 0, this->numTriangles);

   
	return 0;
}




/******************************************************************/
/*
Purpose: sets the surface scaling factor


Descripton: set the position of hte surfce.  The srufce should be 
translated so the the vertex at (0,?,0) is moved to 
Return:

*/





int meshSurface::setScale(float scaleX, float scaleY, float scaleZ)
{
	this->mScaleX = scaleX;
	this->mScaleY = scaleY;
	this->mScaleZ = scaleZ;
	return 0;
}






int meshSurface::initEffect(void)
{
	int rc;
	ID3DXBuffer *errors = NULL;		// used to get the errors
	char *shaderFile[1] = {"simpleshader.fx"};
	char *techniceName[2] = {"FlatTechnique","CraterTechnique"};



	D3DCAPS9 caps;

	md3dDev->GetDeviceCaps(&caps);

	// check for shader support of version 3

	if (caps.VertexShaderVersion < D3DVS_VERSION(3,0)) {
		// no support for Vertex Shader version 3
		// do something  use fixed pipline, try version 2.0 etc.
	}

	// check for pixel Shader Support
	if (caps.PixelShaderVersion < D3DVS_VERSION(3,0)) {
		// no support for Vertex Shader version 3
		// do something  use fixed pipline, try version 2.0 etc.
	}


	// load the effect from the file
	rc = D3DXCreateEffectFromFile(md3dDev, shaderFile[0], 0,0,D3DXSHADER_DEBUG | D3DXSHADER_SKIPOPTIMIZATION, 
		0, &pMeshEffect, &errors);

	if (rc) {
				switch(rc) {
			case D3DXERR_INVALIDDATA:
				rc+=0;
				break;
			case D3DERR_INVALIDCALL:
				rc+=0;
				break;
			case E_OUTOFMEMORY :
				rc+=0;
				break;
			default:
				rc+=0;
				break;
		}
	}
	if (errors != NULL) {
		// displays the errors when loading
		MessageBox(0, (char*) errors->GetBufferPointer(),0,0);
	}

	hWorldMat = hViewMat = hProjMat = hCrater = hFlatTechnique = hCraterTechnique = NULL;

	// associate the external variable in the effect file with variable in the C++ program
	hWorldMat = pMeshEffect->GetParameterByName(NULL, "gWorldMat");

	// associate the external variable in the effect file with variable in the C++ program
	hViewMat = pMeshEffect->GetParameterByName(NULL, "gViewMat");

	// associate the external variable in the effect file with variable in the C++ program
	hProjMat = pMeshEffect->GetParameterByName(NULL, "gProjMat");

	// associate the external variable in the effect file with variable in the C++ program
	hTime = pMeshEffect->GetParameterByName(NULL, "time");

	// associate the external variable in the effect file with variable in the C++ program
	hCrater = pMeshEffect->GetParameterByName(NULL, "crater");

	// associate the technique in the effect file with the C++ program
	hFlatTechnique = pMeshEffect->GetTechniqueByName(techniceName[0]);

	// associate the technique in the effect file with the C++ program
	hCraterTechnique = pMeshEffect->GetTechniqueByName(techniceName[1]);




	if (errors != NULL) {
		errors->Release();
	}
	return 0;
}

// creates a crater in the surface
int meshSurface::setCraterPosition(float x, float y, float z)
{
	crater.x = x;
	crater.y = y;
	crater.z = z;
	return 0;
}

int meshSurface::updateCraterPosition(float dx, float dy, float dz)
{
	crater.x += dx;
	crater.y += dy;
	crater.z += dz;
	return 0;
}
