//=============================================================================
// myGame.cpp
//
//Author: Doron Nussbaum (C) 2011 All Rights Reserved.
//-----------------------------------------------------
//
// Purpose: 
//--------------
// a. demonstrates the usage of the gameIO and gameApp framework 
//
//
//
// Description:
//--------------
// A simple application that draws text on the screen.  
// The application draws a text on the screen.  The text shows the text location 
// and the current frame number.  The text scrolls on the screen.
// The text scrolling changes direction when:
// a. the text reaches a boundary of the window
//
// b. the user presses the left mouse button - the left mouse button changes x-direction
// and the right mouse button changes the y-direction
//
// c. The escape key leaves the application
//
//
//
// License
//--------------
//
// Code can be used for instructional and educational purposes and for assignments in the gaming courses at 
// the School of Compuer Science at Carleton University.
// Usage of code for other purposes is not allowed without a given permission by the author.
//
//
// Disclaimer
//--------------
//
// The code is provided as is without any warranty

//=============================================================================


#include "StdAfx.h"
#include "myGame.h"

#include <stdio.h>





myGame::myGame(HINSTANCE hInstance, char* gameName):gameApp(hInstance, gameName)
, x(0), fontCourier(NULL), ground(NULL)
{
}

myGame::~myGame(void)
{
	if (fontCourier != NULL) {
		fontCourier->Release();
		fontCourier = NULL;
	}
	if (ground != NULL) {
		delete(ground);
	}
}

/************************************************************************/

int myGame::updateGameState(long time)
{


	int rc = 0;
	int pollRC = 0;
	D3DXVECTOR3 mouseDelta;

	// add code to update the game state


	// poll the input
	pollRC =  mInput->poll();
	if (pollRC != 0) {
		pollRC += 0;
	}
	
	// check if escape key was pressed
	if (mInput->keyboardPressed(DIK_ESCAPE)) {
		rc = 1;
	} else {
		if (mInput->keyboardPressed(DIK_LEFT)) {
			// move left
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_RIGHT)) {
			// move right
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_UP)) {
			// pull up

			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_DOWN)) {
			// dive

			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_W)) {
			// speed up

			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_S)) {
			// slow down

			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_A)) {
			//roll left
			cam.roll((float)0.2);  //CCW
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_D)) {
			// roll write
			cam.roll((float)-0.2);  //CW
			rc = 0;
		}
	}


	// move the crater around
	static int nc = 0;
	nc++;
	if (nc % 1000 < 500) {
		ground->updateCraterPosition(3,2,2);
	} else {
		ground->updateCraterPosition(-3,-2,-2);
	}





	return(rc);
}

/**********************************************************************/
#if 0
int myGame::updateGameState(long time)
{
	int rc = 0;
	int changeSpeed = 1;

	// add code to update the game state


	// poll the input
	mInput->poll();
	
	// check if escape key was pressed
	if (mInput->keyboardPressed(DIK_ESCAPE)) {
		rc = 1;
	} else {
		if (mInput->keyboardPressed(DIK_LEFT)) {
			// left arrow yaw left
			cam.yaw((float) -changeSpeed); 
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_RIGHT)) {
			cam.yaw(changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_UP)) {
			// pull up
			cam.pitch((float) changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_DOWN)) {
			// dive
			cam.pitch((float) -changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_W)) {
			// move forward
			cam.moveForward((float) changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_W) &
			(mInput->keyboardPressed(DIK_LSHIFT) | mInput->keyboardPressed(DIK_RSHIFT))) {
			
			// move forward fast
			cam.moveForward((float) 3*changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_S)) {
			// move backwards
			cam.moveForward((float) -changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_S) & 
			(mInput->keyboardPressed(DIK_LSHIFT) | mInput->keyboardPressed(DIK_RSHIFT))) {
			// move backwards fast
			cam.moveForward((float) -3*changeSpeed);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_A)) {
			//roll left
			cam.roll((float) changeSpeed);  //CCW
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_D)) {
			// roll write
			cam.roll((float) -changeSpeed);  //CW
			rc = 0;
		}
	}

	// move the camera
	cam.moveForward(cam.getSpeed());



	return(rc);
}

#endif


#if 0
int myGame::updateGameState(long time)
{
	int rc = 0;
	

	// add code to update the game state


	// poll the input
	mInput->poll();
	
	// check if escape key was pressed
	if (mInput->keyboardPressed(DIK_ESCAPE)) {
		rc = 1;
	} else {
		if (mInput->keyboardPressed(DIK_LEFT)) {
			// left arrow yaw left
			cam.yaw((float) -0.2); 
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_RIGHT)) {
			cam.yaw(0.2);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_UP)) {
			// pull up
			cam.pitch((float) 0.2);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_DOWN)) {
			// dive
			cam.pitch((float) -0.2);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_W)) {
			// move forward
			cam.moveForward((float) 0.8);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_W) &
			(mInput->keyboardPressed(DIK_LSHIFT) | mInput->keyboardPressed(DIK_RSHIFT))) {
			
			// move forward fast
			cam.moveForward((float) 3.6);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_S)) {
			// move backwards
			cam.moveForward((float) -0.8);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_S) & 
			(mInput->keyboardPressed(DIK_LSHIFT) | mInput->keyboardPressed(DIK_RSHIFT))) {
			// move backwards fast
			cam.moveForward((float) -3.6);
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_A)) {
			//roll left
			cam.roll((float) 0.2);  //CCW
			rc = 0;
		}
		if (mInput->keyboardPressed(DIK_D)) {
			// roll write
			cam.roll((float) -0.2);  //CW
			rc = 0;
		}
	}

	// move the camera
	cam.moveForward(cam.getSpeed());



	return(rc);
}

#endif
/**************************************************************************/



 // renders all objects
int myGame::renderFrame(int time)
{

	int i = 0;
	int rc;
	static float step = 1;
	float rad = 0;
	D3DXMATRIX worldMat, viewMat, matTransform, matProjection, matScale, matTranslate,  matRotation;


	//// set up the camera location
	//D3DXMatrixLookAtLH(&viewMat, 
	//&D3DXVECTOR3(40.0f, 40.0f, 40.0f), // camera loc
	//&D3DXVECTOR3(150.0f, 0.0f, -150.0f),  // look-at target 
	//&D3DXVECTOR3(0.0f, 1.0f, 0.0f)); // up Veotor
	//D3DXMatrixLookAtLH(&viewMat,&D3DXVECTOR3(50,1000,50), &D3DXVECTOR3(500,0,500), &D3DXVECTOR3(0,1,0));


	// set the camera matrix
	cam.getViewMatrix(&viewMat);  // nuss1
	// inform direct3d about the view transformation
#ifdef NUSS_SHADERS
	// set the view matrix in the gameObject
	gameObject::setViewMat(&viewMat);

#else
	rc = md3dDev->SetTransform(D3DTS_VIEW,&viewMat);
#endif


	//// set up the projection transformation
	//D3DXMatrixPerspectiveFovLH(&matProjection,  
	//	D3DXToRadian(80),			// field of view
	//	400/300, 					// aspect ratio
	//	1.0f, 1500.0f); 			// near and far planes

	getProjMat(&matProjection);
#ifdef NUSS_SHADERS
	// set the view matrix in the gameObject
	gameObject::setProjMat(&matProjection);
#else	// inform direc3d about the proejctions tranfsormation
	md3dDev->SetTransform(D3DTS_PROJECTION, &matProjection); 
#endif



    // clear the window to a deep blue
	rc = md3dDev->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(50, 50, 50), 1.0f, 0);
	if (rc != D3D_OK) {
		i++;
	}
	md3dDev->BeginScene();    // begins the 3D scene


	ground->render(time);

    md3dDev->EndScene();    // ends the 3D scene

    rc = md3dDev->Present(NULL, NULL, NULL, NULL);   // displays the created frame on the screen
	if (rc != D3D_OK) {
		i++;
	}

	 return 0;
 }



int myGame::initGame(void)
{

	int rc = 0;
	//cube v;

	// set the intial location of the camera
	cam.setCamera(D3DXVECTOR3(500,450,300), D3DXVECTOR3(700,0,700), D3DXVECTOR3(0,1,0));
	cam.setCamera(D3DXVECTOR3(100,750,-300), D3DXVECTOR3(700,0,700), D3DXVECTOR3(0,1,0));
	//cam.setCamera(D3DXVECTOR3(0,0,1), D3DXVECTOR3(0,0,-1), D3DXVECTOR3(0,1,0));

	// initialize the projection matrix
	setProj(1.0,5000.0,D3DXToRadian(80),((float) this->mWndWidth)/this->mWndHeight);

	// add code to initialize the game
	//ground = new meshSurface(100, 100, 20,20);
	ground = new meshSurface(100, 100, 20, 20);
	ground->setCraterPosition(300,300,300);
	ground->setScale(1.0,1.0,1.0);
	if (ground == NULL) {
		// error
		goto err;
		rc = 0;
	}

	// create a font

	rc = D3DXCreateFont(md3dDev,
						10, // height of font
						0,	// use the default width
						FW_MEDIUM+50,
						//FW_NORMAL,	// normal font weight
						1, // no Mipmap
						FALSE, // italic
						DEFAULT_CHARSET, // default character set
						OUT_DEFAULT_PRECIS, // default precision
						DEFAULT_QUALITY, // default quality
						DEFAULT_PITCH ||FF_DONTCARE, // more defaults...
						"Courier",	// typeface Courier"
						&fontCourier); 


	if (rc != S_OK) {
		rc = 1;
	}

	x = this->getWndWidth() / 2;
	y = this->getWndHeight() / 2;
	dx = 1;
	dy = 1;

	textBox.top = 0;
	textBox.left = 0;
	textBox.bottom = 50;
	textBox.right = 280;


	// get the dimension of the window

	// initialize a cube
	


	return 0;

err:
	return(rc);
}

// sets the matrices of the view points and the projection
int myGame::setMatrices(void)
{

	// get the input


	// process input

	// if roll then change the up vector rotation

	// if pitch then change the lookAtVector rise angle

	// if yaw then change the looAtVector


	return 0;
}
